unit ICInstall;

interface

	function SaveICComponent: OSErr;
	function InstallICComponent: OSErr;
	procedure ConfirmInstallICComponent;

implementation

	uses
		Folders, Components, 

		ICTypes, ICCAPI, 

		ICMiscSubs, ICGlobals, ICStandardFile;

	const			(* why aren't these in Components.p??? *)
		registerCmpGlobal = 1;
		registerCmpNoDuplicates = 2;
		registerCompAfter = 4;

	function IsRegistered: boolean;
		var
			inst: ComponentInstance;
			junk: OSErr;
	begin
		inst := nil;
		if has_components then begin
			inst := OpenDefaultComponent(internetConfigurationComponentType, internetConfigurationComponentSubType);
			if inst <> nil then begin
				junk := CloseComponent(inst);
			end; (* if *)
		end; (* if *)
		IsRegistered := (inst <> nil);
	end; (* IsRegistered *)

	function IsInstalled (var where: FSSpec): boolean;
		var
			err: OSErr;
			ndx: integer;
			found: boolean;
			cpb: CInfoPBRec;
			info: FInfo;
	begin
		err := FindFolder(kOnSystemDisk, kExtensionFolderType, true, where.vRefNum, where.parID);
		if err = noErr then begin
			where.name := GetAString(128, 12);
			err := HGetFInfo(where.vRefNum, where.parID, where.name, info);
			if err <> noErr then begin
				found := false;
				ndx := 1;
				repeat
					with cpb do begin
						ioNamePtr := @where.name;
						ioVRefNum := where.vRefNum;
						ioDirID := where.parID;
						ioFDirIndex := ndx;
					end; (* with *)
					err := PBGetCatInfoSync(@cpb);
					if err = noErr then begin
						found := (cpb.ioFlFndrInfo.fdType = 'thng') and (cpb.ioFlFndrInfo.fdCreator = ICcreator);
					end; (* if *)
					ndx := ndx + 1;
				until found or (err <> noErr);
				if not found then begin
					where.name := GetAString(128, 12);
					err := fnfErr;
				end; (* if *)
			end; (* if *)
		end; (* if *)
		IsInstalled := (err = noErr);
	end; (* IsInstalled *)

	function GetInstalledVersion (fss: FSSpec; var vers: longint): OSErr;
		var
			ref: integer;
			err: OSErr;
			versh: VersRecHndl;
	begin
		ref := HOpenResFile(fss.vRefNum, fss.parID, fss.name, fsRdPerm);
		err := ResError;
		if err = noErr then begin
			versh := VersRecHndl(Get1Resource('vers', 1));
			if versh = nil then begin
				err := resNotFound;
			end; (* if *)
			if err = noErr then begin
				vers := versh^^.numericVersion.version;
			end; (* if *)
			CloseResFile(ref);
		end; (* if *)
		GetInstalledVersion := err;
	end; (* GetInstalledVersion *)

	function SaveComponentToFile (fss: FSSpec): OSErr;
		var
			err: OSErr;
			thng: Handle;
			rref: integer;
			junk: OSErr;
	begin
		err := noErr;
		thng := GetResource('ThNg', 128);
		if thng = nil then begin
			err := resNotFound;
		end; (* if *)
		if err = noErr then begin
			HNoPurge(thng);
			junk := HCreate(fss.vRefNum, fss.parID, fss.name, ICcreator, 'thng');
			err := HOpenRF(fss.vRefNum, fss.parID, fss.name, fsRdWrPerm, rref);
			if err = noErr then begin
				err := SetEOF(rref, GetHandleSize(thng));
				if err = noErr then begin
					err := FSWriteQ(rref, GetHandleSize(thng), thng^);
				end; (* if *)
				junk := FSClose(rref);
			end; (* if *)
			HPurge(thng);
		end; (* if *)
		SaveComponentToFile := err;
	end; (* SaveComponentToFile *)

	function RegisterFile (fss: FSSpec): OSErr;
		var
			ref: integer;
			err: OSErr;
			result: longint;
			err2: OSErr;
	begin
		ref := HOpenResFile(fss.vRefNum, fss.parID, fss.name, fsRdPerm);
		err := ResError;
		if err = noErr then begin
			result := RegisterComponentResourceFile(ref, registerCmpGlobal);
			if result > 0 then begin
				err := noErr;
			end
			else begin
				err := result;
			end; (* if *)
			CloseResFile(ref);
			err2 := ResError;
			if err = noErr then begin
				err := err2;
			end; (* if *)
		end; (* if *)
		RegisterFile := err;
	end; (* RegisterFile *)

	function SaveICComponent: OSErr;
		var
			junklong: longint;
			err: OSErr;
			fss: FSSpec;
	begin
		err := ICStandardPutFile(GetAString(128, 13), GetAString(128, 12), fss);
		if err = noErr then begin
			err := SaveComponentToFile(fss);
		end;
		SaveICComponent := err;
	end; (* SaveICComponent *)

	function InstallICComponent: OSErr;
		var
			err: OSErr;
			where: FSSpec;
			prompt: Str255;
			desc: Str255;
			vers: longint;
			ndx: integer;
			junk: integer;
	begin
		err := noErr;
		if IsInstalled(where) then begin
			prompt := GetAString(128, 9);
			desc := '';
			err := GetInstalledVersion(where, vers);
			if err = noErr then begin
				if vers < app_version.numericVersion.version then begin
					desc := GetAString(128, 10);
				end
				else if vers > app_version.numericVersion.version then begin
					desc := GetAString(128, 11);
				end; (* if *)
				ndx := Pos('^1', prompt);
				Delete(prompt, ndx, 2);
				Insert(desc, prompt, ndx);
				ParamText(prompt, '', '', '');
				InitCursor;
				if CautionAlert(141, nil) <> ok then begin
					err := userCanceledErr;
				end; (* if *)
			end; (* if *)
		end; (* if *)
		if err = noErr then begin
			err := SaveComponentToFile(where);
		end; (* if *)
		if err = noErr then begin
			if not IsRegistered and has_components then begin
				err := RegisterFile(where);
			end; (* if *)
		end; (* if *)
		if err = noErr then begin
			InitCursor;
			junk := NoteAlert(142, nil);
		end; (* if *)
		InstallICComponent := err;
	end; (* InstallICComponent *)

	procedure ConfirmInstallICComponent;
		var
			junkfss: FSSpec;
			response: longint;
	begin
		(* Only offer to install the component if they have Component Manager *)
		(* Everyone with Component Manager should use the component and hence be pestered. *)
		(* Everyone without Component Manager won't appreciate the hassles. *)
		if has_components then begin
			if not IsRegistered then begin
				if not IsInstalled(junkfss) then begin
					ParamText(GetAString(128, 8), '', '', '');
					InitCursor;
					if NoteAlert(141, nil) = ok then begin
						DisplayError(acInstallComponent, InstallICComponent);
					end; (* if *)
				end; (* if *)
			end; (* if *)
		end; (* if *)
	end; (* ConfirmInstallICComponent *)

end. (* ICInstall *)